sealed class ShiftsEvent {
  const ShiftsEvent();
}

class ShiftsInitialEvent extends ShiftsEvent {
  const ShiftsInitialEvent();
}

class ShiftsTabChangedEvent extends ShiftsEvent {
  final int tabIndex;

  const ShiftsTabChangedEvent({required this.tabIndex});
}

class LoadTabShiftEvent extends ShiftsEvent {
  final int status;

  const LoadTabShiftEvent({required this.status});
}

class LoadMoreShiftEvent extends ShiftsEvent {
  final int status;

  const LoadMoreShiftEvent({required this.status});
}

class ReloadMissingBreakShift extends ShiftsEvent {

  const ReloadMissingBreakShift();
}
